﻿using System;
using System.Windows.Forms;
using System.Drawing;
using System.IO;

namespace SSRSSEditor
{
    public partial class MainForm : Form
    {
        RssDocument rssDoc;
        bool ignoreChanges;

        public MainForm()
        {
            InitializeComponent();
            System.Drawing.Color prev = txtLink.BackColor;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
			Save();
		}

		private bool Save()
		{
			if (!CheckRequiredField(txtTitle, "Title"))
                return false;
            if(!CheckRequiredField(txtDesc, "Description"))
				return false;
            if(!CheckRequiredField(txtLink, "Link"))
				return false;

            if (!UrlUtils.ValidateUrl(txtLink, true))
				return false;
            if (!UrlUtils.ValidateUrl(txtImage, true))
			{
                txtLink.Focus();
				return false;
			}

			if (!rssDoc.Save(filePath.Text))
				return false;

            UpdateSaveButton();
			return true;
        }

        private bool CheckRequiredField(TextBox box, string name)
        {
            if (!box.Text.IsNullOrEmpty())
                return true;
            Program.MsgBox("A "+name+" is required.", MessageBoxButtons.OK);
            box.Focus();
            return false;
        }

		private bool CanProceed(string eventDesc)
		{
			if ((rssDoc == null) || !rssDoc.IsDirty)
				return true;	// Nothing to save

			DialogResult rslt = Program.MsgBox("Save edits before "+eventDesc+"?", MessageBoxButtons.YesNoCancel);
			if (rslt == DialogResult.Yes)
				return Save();

			if (rslt == DialogResult.Cancel)
				return false;	// Don't abandon edits

			return true;	// Abandon edits
		}

        private void btnBrowse_Click(object sender, EventArgs e)
        {
			if (!CanProceed("loading new feed"))
				return;

            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Title = "Select Feed File to Edit";
            openFileDialog.InitialDirectory = RegistryUtils.GetFileDir();
            openFileDialog.Filter = "RSS files (*.rss;*.xml)|*.rss;*.xml|All files (*.*)|*.*";
            openFileDialog.FilterIndex = 1;
            openFileDialog.RestoreDirectory = true;
            openFileDialog.CheckFileExists = false;

        Loop:
            if (openFileDialog.ShowDialog() != DialogResult.OK)
                return;

            DiskFile file = new DiskFile(openFileDialog.FileName);

            if (!file.Exists())
            {
                DialogResult rslt = Program.MsgBox("Create " + file.FileName + "?", MessageBoxButtons.YesNo);
                if (rslt != DialogResult.Yes)
                    goto Loop;
                RssDocument emptyRssDoc = new RssDocument();
                emptyRssDoc.Save(file.FullPath);
            }

			LoadFeed(file.FullPath);
        }

        private void FillUI(RssDocument rssDoc)
        {
            ignoreChanges = true;

            txtTitle.Clear();
            txtDesc.Clear();
            txtLink.Clear();
            txtEmail.Clear();
            txtImage.Clear();
            itemList.Items.Clear();

            if (rssDoc != null)
            {
                txtTitle.Text = rssDoc.Title;
                txtDesc.Text = rssDoc.Description;
                txtLink.Text = rssDoc.Link;
                txtEmail.Text = rssDoc.Email;
                txtImage.Text = rssDoc.Image;

                RssItem[] items = rssDoc.Items;
                foreach (RssItem item in items)
                    itemList.Items.Add(item);
            }

            ignoreChanges = false;
        }

        private void EnableControls()
        {
            bool enbl = (rssDoc != null);
            txtTitle.Enabled = enbl;
            txtDesc.Enabled = enbl;
            txtLink.Enabled = enbl;
            btnOpenLink.Enabled = enbl;
            UpdateLink(txtLink, btnOpenLink);

            txtEmail.Enabled = enbl;

            txtImage.Enabled = enbl;
            btnImageView.Enabled = enbl;
            UpdateLink(txtImage, btnImageView);

            itemList.Enabled = enbl;

            UpdateSaveButton();
            UpdateListButtons();
        }

        private void UpdateSaveButton()
        {
            bool enbl = false;
            if (rssDoc != null)
                enbl = rssDoc.IsDirty;
            btnSave.Enabled = enbl;
        }

        private void UpdateListButtons()
        {
            bool hasItems = (itemList.Items.Count > 0);
            bool hasSel = (itemList.SelectedIndex >= 0);

            btnItemEdit.Enabled = hasItems && hasSel;
            btnItemDelete.Enabled = hasItems && hasSel;
            btnView.Enabled = hasItems && hasSel;

            btnItemAdd.Enabled = (rssDoc != null);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            EnableControls();
            filePath.Text = "";
            this.FormBorderStyle = FormBorderStyle.FixedSingle; // Because we're not in dialog mode
        }

        private void txtTitle_TextChanged(object sender, EventArgs e)
        {
            if (!ignoreChanges)
            {
                rssDoc.Title = txtTitle.Text;
                UpdateSaveButton();
            }
        }

        private void txtDesc_TextChanged(object sender, EventArgs e)
        {
            if (!ignoreChanges)
            {
                rssDoc.Description = txtDesc.Text;
                UpdateSaveButton();
            }
        }

        private void UpdateLink(TextBox edit, Button btn)
        {
            btn.Enabled = UrlUtils.ValidateUrl(edit, false) && !edit.Text.IsNullOrEmpty();
        }

        private void txtLink_TextChanged(object sender, EventArgs e)
        {
            UpdateLink(txtLink, btnOpenLink);
            if (!ignoreChanges)
            {
                rssDoc.Link = txtLink.Text;
                UpdateSaveButton();
            }
        }

        private void btnOpenLink_Click(object sender, EventArgs e)
        {
            if (!txtLink.Text.IsNullOrEmpty() && UrlUtils.ValidateUrl(txtLink, true))
                UrlUtils.OpenInBrowser(txtLink.Text);
        }

        private void txtEmail_TextChanged(object sender, EventArgs e)
        {
            if (!ignoreChanges)
            {
                rssDoc.Email = txtEmail.Text;
                UpdateSaveButton();
            }
        }

        private void txtImage_TextChanged(object sender, EventArgs e)
        {
            UpdateLink(txtImage, btnImageView);
            if (!ignoreChanges)
            {
                rssDoc.Image = txtImage.Text;
                UpdateSaveButton();
            }
        }

        private void btnImageView_Click(object sender, EventArgs e)
        {
            if (!txtImage.Text.IsNullOrEmpty() && UrlUtils.ValidateUrl(txtImage, true))
                UrlUtils.OpenInBrowser(txtImage.Text);
        }

        private void itemList_SelChange(object sender, EventArgs e)
        {
            UpdateListButtons();
        }

        private void btnItemEdit_Click(object sender, EventArgs e)
        {
            EditSelectedItem();
        }

        private void EditSelectedItem()
        {
            RssItem item = GetSelectedItem();
            if (item != null)
            {
                EditItemForm form = new EditItemForm(ref item, false);
                if (DialogResult.OK == form.ShowDialog())
                {
                    // Need to replace the item with itself so the list will redraw with updated data
                    itemList.Items[itemList.SelectedIndex] = item;
                }
            }
        }

        private void btnItemDelete_Click(object sender, EventArgs e)
        {
            RssItem item = GetSelectedItem();
            if (item != null)
            {
                rssDoc.DeleteItem(item);
                itemList.Items.RemoveAt(itemList.SelectedIndex);
                UpdateSaveButton();
            }
        }

        private void btnItemAdd_Click(object sender, EventArgs e)
        {
            RssItem item = new RssItem();
            EditItemForm form = new EditItemForm(ref item, true);
            if (DialogResult.OK == form.ShowDialog())
            {
                itemList.Items.Add(item);
                rssDoc.AddItem(item);
                UpdateSaveButton();
            }
        }

        private void btnView_Click(object sender, EventArgs e)
        {
            RssItem item = GetSelectedItem();
            if (item != null)
                UrlUtils.OpenInBrowser(item.link);
        }

        private RssItem GetSelectedItem()
        {
            RssItem item = null;
            if (itemList.SelectedIndex >= 0)
            {
                item = itemList.Items[itemList.SelectedIndex] as RssItem;
            }
            return item;
        }

        private void itemList_DoubleClick(object sender, EventArgs e)
        {
            EditSelectedItem();
        }

		private void btnNew_Click(object sender, EventArgs e)
		{
			if (!CanProceed("creating new feed"))
				return;

			SaveFileDialog saveFileDialog = new SaveFileDialog();
			saveFileDialog.Title = "Select New Feed Filename";
			saveFileDialog.InitialDirectory = RegistryUtils.GetFileDir();
			saveFileDialog.Filter = "RSS files (*.rss;*.xml)|*.rss;*.xml|All files (*.*)|*.*";
			saveFileDialog.FilterIndex = 1;
			saveFileDialog.RestoreDirectory = true;
			saveFileDialog.CheckFileExists = false;
			saveFileDialog.OverwritePrompt = true;

			if (saveFileDialog.ShowDialog() != DialogResult.OK)
				return;

			DiskFile file = new DiskFile(saveFileDialog.FileName);

			RssDocument emptyRssDoc = new RssDocument();
			emptyRssDoc.Save(file.FullPath);

			LoadFeed(file.FullPath);
		}

		private void LoadFeed(string fullPath)
		{
			filePath.Text = fullPath;

			string path = Path.GetDirectoryName(fullPath);
			RegistryUtils.SetFileDir(path);

			FillUI(null);

			rssDoc = new RssDocument();
			if (rssDoc.Load(fullPath))
			{
				FillUI(rssDoc);
			}
			else
			{
				rssDoc = null;
			}

			EnableControls();
		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			e.Cancel = !CanProceed("closing");
		}

    }
}
