﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Xml;

namespace SSRSSEditor
{
    // Spec: http://cyber.law.harvard.edu/rss/rss.html
    public class RssDocument
    {
        private string _title, _description, _link, _email, _image;  // Channel values

        List<RssItem> _items;

        public bool IsDirty {get; private set;}

        public RssDocument()
        {
            IsDirty = false;
        }

        public bool Load(string file)
        {
            XmlDocument xDoc = new XmlDocument();
            try
            {
                xDoc.Load(file);
                IsDirty = false;
            }
            catch (Exception e)
            {
                Program.MsgBox("Failed to load file (XmlDocument exception):\n" + e.Message, MessageBoxButtons.OK);
                return false;
            }

            // Ignore original pubDate - if we save we'll replace it
            _title = GetNodeInnerText(xDoc, "/rss/channel/title");
            _description = GetNodeInnerText(xDoc, "/rss/channel/description");
            _link = GetNodeInnerText(xDoc, "/rss/channel/link");
            _email = GetNodeInnerText(xDoc, "/rss/channel/managingEditor");
            if(_email.IsNullOrEmpty())
                _email = GetNodeInnerText(xDoc, "/rss/channel/webMaster");
            _image = GetNodeInnerText(xDoc, "/rss/channel/image/url");

            _items = new List<RssItem>();
            XmlNodeList nodes = xDoc.SelectNodes("/rss/channel/item");
            foreach (XmlNode node in nodes)
                _items.Add(new RssItem(node));

            return true;
        }

        public bool Save(string path)
        {
            XmlDocument xDoc = new XmlDocument();
            
            xDoc.AppendChild(xDoc.CreateXmlDeclaration("1.0", "UTF-8", null));

            XmlNode rss = xDoc.CreateElement("rss");
            xDoc.AppendChild(rss);
            rss.SetAttribute("version", "2.0");

            XmlNode channel = rss.AddChildNode("channel");
            channel.AddChildNode("generator").InnerText = "SSRSSEditor";
            channel.AddChildNode("pubDate").InnerText = DateTime.Now.ToRFC1123String();
            channel.AddChildNode("title").InnerText = _title;
            channel.AddChildNode("description").InnerText = _description;
            channel.AddChildNode("link").InnerText = _link;
            channel.AddChildNode("language").InnerText = "en";

            // Optional
            if (!_email.IsNullOrEmpty())
            {
                channel.AddChildNode("managingEditor").InnerText = _email;
                channel.AddChildNode("webMaster").InnerText = _email;
            }

            // Optional
            if (!_image.IsNullOrEmpty())
            {
                XmlNode image = channel.AddChildNode("image");
                image.AddChildNode("url").InnerText = _image;
                image.AddChildNode("title").InnerText = _title;
                image.AddChildNode("link").InnerText = _link;
            }

			if ((_items != null) && (_items.Count > 0))
			{
				foreach (RssItem item in _items)
					item.AppendTo(channel);
			}

            try
            {
                xDoc.Save(path);
                IsDirty = false;
                return true;
            }
            catch (Exception e)
            {
                Program.MsgBox("Failed to save file (XmlDocument exception):\n" + e.Message, MessageBoxButtons.OK);
            }

            return false;
        }

        public void AddItem(RssItem itemToAdd)
        {
            _items.Add(itemToAdd);
            IsDirty = true;
        }

        public void DeleteItem(RssItem itemToDelete)
        {
            _items.Remove(itemToDelete);
            IsDirty = true;
        }

        public string Title
        {
            get { return _title;  } 
            set { _title=value; IsDirty=true;  }
        }
        public string Description
        {
            get { return _description;  } 
            set { _description=value; IsDirty=true;  }
        }
        public string Link
        {
            get { return _link;  } 
            set { _link=value; IsDirty=true;  }
        }
        public string Email
        {
            get { return _email;  } 
            set { _email=value; IsDirty=true;  }
        }
        public string Image
        {
            get { return _image;  } 
            set { _image=value; IsDirty=true;  }
        }

        public RssItem[] Items
        {
            get 
            { /*
                List<RssItem> items = new List<RssItem>();

                XmlNodeList nodes = xDoc.GetElementsByTagName("item");
                foreach (XmlNode node in nodes)
                    items.Add(new RssItem(node));
                */
                return _items.ToArray();
            }
        }

        //protected string GetChannelItemText(string item)
        //{
        //    return GetNodeInnerText("/rss/channel/"+item); 
        //}
        
        //protected void SetChannelItemText(string item, string value)
        //{
        //    XmlNode channel = GetChannelNode();
        //    channel.GetOrCreateChildNode(item).InnerText = value;
        //    IsDirty = true;
        //}

        //protected XmlNode GetRssNode()
        //{
        //    XmlNode rss = xDoc.SelectSingleNode("/rss");
        //    if(rss == null)
        //    {
        //        rss = xDoc.CreateElement("rss");
        //        xDoc.AppendChild(rss);
        //        rss.SetAttribute("version", "2.0");
        //    }
        //    return rss;
        //}

        //protected XmlNode GetChannelNode()
        //{
        //    XmlNode rss = GetRssNode();
        //    return rss.GetOrCreateChildNode("channel");
        //}

        protected string GetNodeInnerText(XmlDocument xDoc, string xpath)
        {
            string value = "";
            XmlNode node = xDoc.SelectSingleNode(xpath);
            if (node != null)
                value = node.InnerText;
            return value;
        }

    }

    public class RssItem
    {
        public string title, description, link, guid;
        DateTime pubDate;

        public RssItem()
        {
            guid = Guid.NewGuid().ToString();
            UpdatePubDate();
        }

        public RssItem(XmlNode node)
        {
            title = node.GetOrCreateChildNode("title").InnerText;
            description = node.GetOrCreateChildNode("description").InnerText;
            link = node.GetOrCreateChildNode("link").InnerText;
            guid = node.GetOrCreateChildNode("guid").InnerText;
            try
            {
                pubDate = DateTime.Parse(node.GetOrCreateChildNode("pubDate").InnerText);
            }
            catch
            {
                // Parse() probably threw a DateTimeFormat exception.
                pubDate = DateTime.Now; // What else could we do?
            }
        }

        // Used by listbox for auto object rendering
        public override string ToString()
        {
            return pubDate.ToString("D") + " " + pubDate.ToString("t") + "\t" + title;
        }

        public void UpdatePubDate()
        {
            pubDate = DateTime.Now;
        }

        public XmlNode AppendTo(XmlNode parent)
        {
            XmlNode xItem = parent.OwnerDocument.CreateElement("item");
            parent.AppendChild(xItem);

            xItem.AddChildNode("title").InnerText = title;
            xItem.AddChildNode("description").InnerText = description;
            xItem.AddChildNode("pubDate").InnerText = pubDate.ToRFC1123String();
            xItem.AddChildNode("link").InnerText = link;
            XmlNode xGuid = xItem.AddChildNode("guid");
            xGuid.InnerText = guid;
            xGuid.SetAttribute("isPermaLink", "false");

            return xItem;
        }
    }

}
