﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using System.Xml;
using Microsoft.Win32;

namespace SSRSSEditor
{
    #region StringUtils
    public static class StringUtils
    {
        public static bool IsNullOrEmpty(this string value)
        {
            return string.IsNullOrEmpty(value);
        }

        public static string StripNonUppercase(this string source)
        {
            string result = "";
            if (!source.IsNullOrEmpty())
            {
                foreach (char c in source)
                {
                    if (Char.IsUpper(c))
                        result += c;
                }
            }
            return result;
        }


    }
    #endregion

    #region StringUtils
    public static class RegistryUtils
    {
        const string APPSUBKEY = @"Software\dlchambers\SSRSSEditor";
        const string APPVAL_FILEDIR = @"filedir";
        const string DEFAULT_FILEDIR = @"c:\Program Files";

        public static string GetFileDir()
        {
            string dir = null;
            RegistryKey oRegKey = Registry.CurrentUser.CreateSubKey(APPSUBKEY);
            if(oRegKey != null)
            {
                dir = (string)oRegKey.GetValue(APPVAL_FILEDIR);
                oRegKey.Close();
            }

            if(dir.IsNullOrEmpty())
                dir = DEFAULT_FILEDIR;

            return dir;
        }

        public static void SetFileDir(string dir)
        {
            RegistryKey oRegKey = Registry.CurrentUser.CreateSubKey(APPSUBKEY);
            if (oRegKey != null)
            {
                oRegKey.SetValue(APPVAL_FILEDIR, dir);
                oRegKey.Close();
            }
        }

        public static string GetDefaultBrowserPath()
        {
            string key = @"htmlfile\shell\open\command";
            RegistryKey registryKey = Registry.ClassesRoot.OpenSubKey(key, false);
            return ((string)registryKey.GetValue(null, null)).Split('"')[1];
        }
    }
    #endregion

    #region XmlUtils
    public static class XmlUtils
    {
        public static void SetAttribute(this XmlNode node, string name, string value)
        {
            XmlAttribute attr = node.OwnerDocument.CreateAttribute(name);
            attr.Value = value;
            node.Attributes.Append(attr);
        }

        public static XmlNode GetOrCreateChildNode(this XmlNode parent, string childName)
        {
            XmlNode child = parent.SelectSingleNode(childName);
            if (child == null)
            {
                child = parent.OwnerDocument.CreateElement(childName);
                parent.AppendChild(child);
            }
            return child;
        }

        public static XmlNode AddChildNode(this XmlNode parent, string childName)
        {
            XmlNode child = parent.OwnerDocument.CreateElement(childName);
            parent.AppendChild(child);
            return child;
        }


    }
    #endregion

    #region DiskFile
    public class DiskFile
    {
        public string FullPath;

        public DiskFile(string path)
        {
            FullPath = path;
        }

        public string FileName
        {
            get
            {
                string[] parts = FullPath.Split(new char[] { '\\' });
                return parts.LastOrDefault();
            }
            //set
            //{
            //}
        }

        public string Path
        {
            get
            {
                int idx = FullPath.LastIndexOf('\\');
                if (idx > 0)
                {
                    return FullPath.Substring(0, idx);
                }
                return FullPath;
            }
            //set
            //{
            //}
        }

        public bool Exists()
        {
            return File.Exists(FullPath);
        }
    }
    #endregion

    #region UrlUtils
    public static class UrlUtils
    {
        public static void OpenInBrowser(string url)
        {
            // open URL in separate instance of default browser
            Process p = new Process();
            p.StartInfo.FileName = RegistryUtils.GetDefaultBrowserPath();
            p.StartInfo.Arguments = url;
            p.Start();
        }

        public static bool ValidateUrl(TextBox box, bool showMessageOnError)
        {
            bool ok = true;

            if (!box.Text.IsNullOrEmpty())
            {
                try
                {
                    Uri p = new Uri(box.Text);
                    ok = (p.Scheme.ToLower() == "http");
                }
                catch (Exception e)
                {
                    ok = false;
                    if (showMessageOnError)
                    {
                        Program.MsgBox("URL format error:\n" + e.Message, MessageBoxButtons.OK);
                        box.Focus();
                    }
                }
            }
            box.BackColor = ok ? (box.Enabled?SystemColors.Window:SystemColors.Control) : System.Drawing.Color.Pink;
            return ok;
        }

    }
    #endregion

    #region TimeUtils
    public static class TimeUtils
    {
        // Format per IETF RFC1233
        // The inclusion of 'ToUniversalTime()' is *critical*
        public static string ToRFC1123String(this DateTime dt)
        {
            return dt.ToUniversalTime().ToString("r");
        }
    }
    #endregion
}
